use crate::auto_events::AutoEvent;
use std::io::prelude::*;
use serde::{Serialize, Deserialize};

#[derive(Serialize, Deserialize, PartialEq, Debug)]
pub enum AvailableAutoEventType {
    Speed = 1,
    RPM = 2,
    FuelLevel = 3,
    GPSLocation = 4,
    Throttle = 5
}

#[derive(Serialize, Deserialize, PartialEq, Debug)]
pub struct FileHeader {
    magic_number: u32,
    version: u8,
    event_types: Vec<AvailableAutoEventType>
}

#[derive(Serialize, Deserialize, PartialEq, Debug)]
pub struct FileContents {
    header: FileHeader,
    records: Vec<AutoEvent>
}

pub struct File {
    file_handle: std::fs::File,
    contents: FileContents
}

impl File {
    pub fn create(path: &str) -> ::anyhow::Result<Self> {
        let new_file = File {
            file_handle: std::fs::File::create_new(path)?,
            contents: FileContents {
                header: FileHeader {
                    magic_number: 0x0BD2_2018,
                    version: 1,
                    event_types: vec![
                        AvailableAutoEventType::Speed,
                        AvailableAutoEventType::RPM,
                        AvailableAutoEventType::FuelLevel,
                        AvailableAutoEventType::GPSLocation,
                        AvailableAutoEventType::Throttle,
                    ]
                },
                records: vec![]
            }
        };

        return Ok(new_file);
    }

    pub fn open(path: &str) -> ::anyhow::Result<Self> {
        println!("0");
        let file_bytes: Vec<u8> = std::fs::read(path)?;
        println!("1");
        let handle = std::fs::OpenOptions::new().read(true).write(true).open(path)?;
        println!("2");

        //let magic_number = bincode::deserialize::<u32>(&file_bytes[0..3])?;
        //let version = bincode::deserialize::<u8>(&file_bytes[4..4])?;
        println!("3");

        //if magic_number != 0x0BD2_2018 {
        //    return Err(anyhow::anyhow!("Bad magic number. Is this an obt file?"));
        //}
        //

        let contents = bincode::deserialize::<FileContents>(file_bytes.as_slice())?;
        
        if contents.header.version != 1 {
            return Err(anyhow::anyhow!("File version {} is unsupported. We only support file version 1.", contents.header.version));
        }

        Ok(Self {
            file_handle: handle,
            contents
        })
    }

    pub fn add_record(&mut self, record: AutoEvent) {
        self.contents.records.push(record);
    }

    pub fn write(&mut self) -> ::anyhow::Result<()> {
        self.file_handle.set_len(0)?;

        let new_contents = bincode::serialize::<FileContents>(&self.contents)?;

        self.file_handle.write_all(&new_contents)?;

        self.file_handle.flush()?;

        Ok(())
    }
    
    pub fn write_json(&mut self) -> ::anyhow::Result<()> {
        println!("$");
        self.file_handle.set_len(0)?;

        let new_contents = serde_json::to_string::<FileContents>(&self.contents)?;

        self.file_handle.write_all(&new_contents.as_bytes())?;

        self.file_handle.flush()?;

        Ok(())
    }
}
